#ifndef FIGHT_PLAYER_H
#define FIGHT_PLAYER_H

#include "state_machine.h"
#include "state.h"
#include "behaviour_entity.h"
#include "animation_sprite.h"
#include "input.h"
#include "vmath.h"
#include "transform.h"
#include "player_input.h"

typedef struct Player {
    Transform transform;
    
    PlayerInput* playerInput;
    
    Vector moveInput;
    int attackInput;
    
    Sprite* sprite;

    AnimationSprite* idle;
    AnimationSprite* walk;
    AnimationSprite* jab_a;
    AnimationSprite* jab_b;

    StateMachine* animationStateMachine;
    
    AnimationSprite* currentAnimation;
} Player;

Player* MakePlayer();
Player* SpawnPlayer(Vector location);
void DestroyPlayer(Player* self);

void PlayerHorizontalInput(Player* self, InputEvent value);
void PlayerVerticalInput(Player* self, InputEvent value);
void PlayerAttackInput(Player* self, InputEvent value);

void PlayerStart(Player* self);
void PlayerUpdate(Player* self, float deltaTime);
void PlayerDraw(Player* self);

impl_Drop_for(Player,
    DestroyPlayer
)

impl_BehaviourEntity_for(Player,
    PlayerStart,
    PlayerUpdate,
    PlayerDraw
)

void PlayerAnimationExit(Player* self);

void PlayerIdleEnter(Player* self);
const State* PlayerIdleUpdate(Player* self, float deltaTime);

DefineState(PlayerIdle, Player,
    PlayerIdleEnter,
    PlayerIdleUpdate,
    PlayerAnimationExit
)

void PlayerWalk_Enter(Player* self);
const State* PlayerWalk_Update(Player* self, float deltaTime);

DefineState(PlayerWalk, Player,
    PlayerWalk_Enter,
    PlayerWalk_Update,
    PlayerAnimationExit
)

void PlayerJabA_Enter(Player* self);
const State* PlayerJabA_Update(Player* self, float deltaTime);

DefineState(PlayerJabA, Player,
    PlayerJabA_Enter,
    PlayerJabA_Update,
    PlayerAnimationExit
)

void PlayerJabB_Enter(Player* self);
const State* PlayerJabB_Update(Player* self, float deltaTime);

DefineState(PlayerJabB, Player,
    PlayerJabB_Enter,
    PlayerJabB_Update,
    PlayerAnimationExit
)

#endif // !FIGHT_PLAYER_H
